@@ -1,16 +1,12 @@
 package DBIx::Class::Storage::DBI::Sybase::ASE::NoBindVars;
 
-use warnings;
-use strict;
-
 use base qw/
   DBIx::Class::Storage::DBI::NoBindVars
   DBIx::Class::Storage::DBI::Sybase::ASE
 /;
 use mro 'c3';
-use List::Util 'first';
-use Scalar::Util 'looks_like_number';
-use namespace::clean;
+use List::Util ();
+use Scalar::Util ();
 
 sub _init {
   my $self = shift;
@@ -21,7 +17,7 @@ sub _init {
 
 sub _fetch_identity_sql { 'SELECT ' . $_[0]->_identity_method }
 
-my $number = sub { looks_like_number $_[0] };
+my $number = sub { Scalar::Util::looks_like_number($_[0]) };
 
 my $decimal = sub { $_[0] =~ /^ [-+]? \d+ (?:\.\d*)? \z/x };
 
@@ -42,7 +38,7 @@ sub interpolate_unquoted {
 
   return $self->next::method(@_) if not defined $value or not defined $type;
 
-  if (my $key = first { $type =~ /$_/i } keys %noquote) {
+  if (my $key = List::Util::first { $type =~ /$_/i } keys %noquote) {
     return 1 if $noquote{$key}->($value);
   }
   elsif ($self->is_datatype_numeric($type) && $number->($value)) {
@@ -72,8 +68,8 @@ Sybase ASE without placeholder support
 
 =head1 DESCRIPTION
 
-If you're using this driver then your version of Sybase or the libraries you
-use to connect to it do not support placeholders.
+If you're using this driver than your version of Sybase, or the libraries you
+use to connect to it, do not support placeholders.
 
 You can also enable this driver explicitly using:
 
@@ -85,7 +81,7 @@ See the discussion in L<< DBD::Sybase/Using ? Placeholders & bind parameters to
 $sth->execute >> for details on the pros and cons of using placeholders.
 
 One advantage of not using placeholders is that C<select @@identity> will work
-for obtaining the last insert id of an C<IDENTITY> column, instead of having to
+for obtainging the last insert id of an C<IDENTITY> column, instead of having to
 do C<select max(col)> in a transaction as the base Sybase driver does.
 
 When using this driver, bind variables will be interpolated (properly quoted of
