@@ -1,16 +1,12 @@
-package # hide from PAUSE
+package # hide from PAUSE 
     DBICTest::Schema::CustomSql;
 
-use warnings;
-use strict;
-
 use base qw/DBICTest::Schema::Artist/;
-use mro 'c3';
 
 __PACKAGE__->table('dummy');
 
 __PACKAGE__->result_source_instance->name(\<<SQL);
-  ( SELECT a.*, cd.cdid AS cdid, cd.title AS title, cd.year AS year
+  ( SELECT a.*, cd.cdid AS cdid, cd.title AS title, cd.year AS year 
   FROM artist a
   JOIN cd ON cd.artist = a.artistid
   WHERE cd.year = ?)
